#include "webgui.h"
#include "mqtt.h"
#include "wifi.h"

#include <ESP8266WiFi.h>
#include <WiFiClient.h>
#include <ESP8266WebServer.h>
#include <DNSServer.h>
#include <ESP8266mDNS.h>
#include <EEPROM.h>
#include "PubSubClient.h"
#include "DHT.h"

ESP8266WebServer server(80);                         // Initialisiere Webserver
WiFiClient wifiClient;
PubSubClient client(wifiClient);
/*
   ------------------------------------------------------------------------------------------------
   [DE] Vervollständige hier die Variablen mit den Infos zu deinem Netzwerk
        wie SSID, MQTT und Standort des Sensors.
   [EN] complete the Variables with Infos for your Homenetwork 
        like SSID, MQTT und location from the Sensor.
   ------------------------------------------------------------------------------------------------
*/

// [DE] 1. Variable WLAN SSID | 2. Variable WLAN Passwort
//.... [EN]  1. Variable WLAN SSID | 2. Variable WLAN Password
const char* var_char_WLAN_SSID = " ";
const char* var_char_WLAN_PASS = " ";

// [DE] Logindaten MQTT-Broker | 1. IP/Hostname | 2. Port | 3. Benutzername | 4. Passwort
//.... [EN] Logincredentials MQTT-Broker | 1. IP/Hostname | 2. Port | 3. Userlogin | 4. Password
const char* var_char_mqtt_server = " ";
const int var_int_mqtt_port = 1883;
const char* var_char_mqtt_user = " ";
const char* var_char_mqtt_password = " mpf123";

// [DE] Sensorstandpunkt angeben: Dabei ist das ganze herachisch für eine eindeutige Zuordnung 
//      Aufgebaut um mehere Gebäude mit mehreren Wohnungen auf einem Broker sauber Idenfizieren
//      zu können. Daher Hausname(Oder Grundstücksname) Wohnung und Raummname usw.
//.... [EN] Specify sensor position: The whole position is written down herachic for a clear assignment.
//          It's usable for around several buildings with several apartments for a clean idenfication 
//          on one MQTT Broker.
const char* var_char_sensor_hausName = "haus01"; // [DE] Bsp: Haus01, ID83374, Bäckerstraße 24b.... [EN] HomeOne, BackerStreet, eg.
const char* var_char_sensor_wohnungsName = "wohnung01"; //[DE] Mannys Wohnung, ID00023, Garten .... [EN] Mannys' Castle, ID0023, Garden
const char* var_char_sensor_raumName = "Kueche"; // [DE] Küche, Flur, Keller .... [EN] Kitchen, Floor, Storage
const char* var_char_sensor_sensorName = "sensor01"; // [DE] Sensor1, FlurSensor, NebenRauchMelder .... [EN] SensorOne, FloorSens, NextToSmokeDetector


/*
   ------------------------------------------------------------------------------------------------
   [DE] Definiere Topics für den MQTT-Server !!! NICHTS MEHR ÄNDERN!
   [EN] Define the Topics für MQTT-Broker !!! DONT CHANGE ANYTHING HERE!
   ------------------------------------------------------------------------------------------------
*/

String Value1 = str_fkt_topicpath() + "/S01_Temperatur";
String Value2 = str_fkt_topicpath() + "/S02_Luftfeuchtigkeit";
String Value3 = str_fkt_topicpath() + "/S03_Helligkeit";
String Value4 = str_fkt_topicpath() + "/S04_Bewegung";
String Value5 = str_fkt_topicpath() + "/D01_Batterie";
String Value6 = str_fkt_topicpath() + "/N01_IP";
String Value7 = str_fkt_topicpath() + "/S05_GAS";
String Value8 = str_fkt_topicpath() + "/S06_CO";
String Value9 = str_fkt_topicpath() + "/A01_Nachtlicht";
String Value10 = str_fkt_topicpath() + "/A02_optischerAlarm";
String Value11 = str_fkt_topicpath() + "/A03_akkustischerAlarm";
String Value12 = str_fkt_topicpath() + "/12_free";
String Value13 = str_fkt_topicpath() + "/D02_AkkuStand";
String Value14 = str_fkt_topicpath() + "/D03_AkkuBetrieb";
String Value15 = str_fkt_topicpath() + "/N02_WLAN-Signal";

/*
   ------------------------------------------------------------------------------------------------
   [DE] Definiere Variablen für den MQTT-Server !!! NICHTS MEHR ÄNDERN!
   [EN] Define the Variables für MQTT-Broker !!! DONT CHANGE ANYTHING HERE!
   ------------------------------------------------------------------------------------------------
*/
String string_var_Temperatur;
String string_var_Luftfeuchtigkeit;
String string_var_Umgebungslicht;
String string_var_Bewegungsmelder;
bool bool_var_NachtLicht_an;
bool bool_var_Notlicht_an;

/*
   ------------------------------------------------------------------------------------------------
   SETUP fÃ¼r die Initialconfig
   ------------------------------------------------------------------------------------------------
*/
void setup() {
  Serial.begin(115200); // FÃ¼r Debuggingzwecke Ausgabe Ã¼ber Serielle Konsole
  Serial.println("*INIT:DO  :Starte Initialisierung");

  bool var_bool_connection_status; // Setzte VAR für Wlanstatus
  var_bool_connection_status = fkt_bool_WLAN_connect(var_char_WLAN_SSID, var_char_WLAN_PASS); // Fülle VAR mit WLANSTATUS

  if (var_bool_connection_status) { //Wenn WLAN dann starte WebServer

    Serial.println("*INIT:DO  :Starte Webserver");
    server.on ( "/", SITE_void_handleRoot );

    const char * headerkeys[] = {"User-Agent", "Cookie"} ;
    size_t headerkeyssize = sizeof(headerkeys) / sizeof(char*);
    server.collectHeaders(headerkeys, headerkeyssize );
    server.begin();
    Serial.println("*INIT:DONE:Starte Webserver");
    Serial.println("*----------------------------------");
  }

  if (var_bool_connection_status) { // Wenn WLAN dann MQTT Dienst
    Serial.println("*INIT:DO  :Baue Verbindung zum MQTT Broker auf");
    client.setServer(var_char_mqtt_server, var_int_mqtt_port);
    client.setCallback(callback);
    while (!client.connected()) {
      Serial.println("Connecting to MQTT...");//
      if (client.connect(WiFi.macAddress().c_str(), var_char_mqtt_user, var_char_mqtt_password )) {
        Serial.println("connected");
      } else {
        Serial.print("failed with state ");
        Serial.print(client.state());
        delay(2000);
      }
    }
    string_var_Temperatur = String(float_fkt_getTemp());
    string_var_Luftfeuchtigkeit = String(float_fkt_getHum());
    string_var_Umgebungslicht = String(float_fkt_getLicht());
    string_var_Bewegungsmelder = String(bool_fkt_getMotion());
    
    
    client.publish(Value1.c_str(), string_var_Temperatur.c_str());
    client.publish(Value2.c_str(), string_var_Luftfeuchtigkeit.c_str());
    client.publish(Value3.c_str(), string_var_Umgebungslicht.c_str() );
    client.publish(Value4.c_str(), string_var_Bewegungsmelder.c_str() );
    client.publish(Value5.c_str(), "90%" );
    client.publish(Value6.c_str(), String(WiFi.localIP()).c_str());
    client.publish(Value7.c_str(), "0" );
    client.publish(Value8.c_str(), "0" );
    client.publish(Value15.c_str(), String(int_fkt_WLANProzent(WiFi.RSSI())).c_str());
    client.publish(Value9.c_str(), "" ); // Nachtlicht
    client.subscribe(Value9.c_str());   // Nachtlicht 
    client.subscribe(Value10.c_str());  // optischer Alarm
    //client.subscribe(Value11.c_str());  // akkustuscher Alarm
    //client.subscribe(Value14.c_str());  // Akkubetrieb
    Serial.println("*INIT:DONE:Baue Verbindung zum MQTT Broker auf");
    Serial.println("*---------------------------------------------");

  }
}

/*
   ------------------------------------------------------------------------------------------------
   BAUE eigenständige Funktionen im Zusammenhang mit dem Root
   ------------------------------------------------------------------------------------------------
*/

bool fkt_bool_is_authentified() {
  Serial.println("Enter is_authentified");
  if (server.hasHeader("Cookie")) {
    Serial.print("Found cookie: ");
    String cookie = server.header("Cookie");
    Serial.println(cookie);
    if (cookie.indexOf("ESPSESSIONID=1") != -1) {
      Serial.println("Authentification Successful");
      return true;
    }
  }
  Serial.println("Authentification Failed");
  return false;
}

String  str_fkt_topicpath(){
  String topicpath = var_char_sensor_hausName ;
  topicpath +="/";
  topicpath += var_char_sensor_wohnungsName ;
  topicpath +="/";
  topicpath += var_char_sensor_raumName ;
  topicpath +="/";
  topicpath += var_char_sensor_sensorName;

  return topicpath;
}

void callback(char* topic, byte* payload, unsigned int length) {
  Serial.println("*INFO:DO  :Payload Topic Check");
  Serial.print("***********Message arrived in topic: ");
  Serial.println(topic);/*
 if (strcmp(topic,"A01_Nachtlicht")) {
   Serial.print("*INFO:DONE:Nachtlicht an");
   bool_var_NachtLicht_an = 1;
 } else {
   Serial.print("*INFO:DONE:Nachtlicht aus");
   bool_var_NachtLicht_an = 0;
 }
 */
  Serial.print("***********Message:");
  for (int i = 0; i < length; i++) {
    Serial.print((char)payload[i]);
  }
 
  Serial.println();
  Serial.println("----------------------------------");
  Serial.println("*INFO:DONE:Payload Topic Check");
  Serial.println("*----------------------------------");
 
}

/*
   ------------------------------------------------------------------------------------------------
   SETUP fÃ¼r die Webseiten
   ------------------------------------------------------------------------------------------------
*/

void SITE_void_handleRoot() {
  Serial.println("*INFO:DO  :Öffne RootWebseite");
  String root_content = Index_Startseite;
  root_content += "<a href=http://'";
  root_content += WiFi.localIP();
  root_content += "'> Startseite </a>";
  root_content.replace("{v}", "Startseite");
  root_content.replace("{h}", String(WiFi.macAddress()));
  // root_content.replace("{setup_topicPathToValue}", str_fkt_topicpath());
  root_content.replace("{infhas}", var_char_sensor_hausName);
  root_content.replace("{infwos}", var_char_sensor_wohnungsName);
  root_content.replace("{infrooms}", var_char_sensor_raumName);
  root_content.replace("{infsens}", var_char_sensor_sensorName);
  root_content.replace("{inftemp}", String(float_fkt_getTemp()));
  root_content.replace("{infhum}", string_var_Luftfeuchtigkeit);
  root_content.replace("{inflit}", String(float_fkt_getLicht()));
  root_content.replace("{infbewg}", string_var_Bewegungsmelder);
  root_content.replace("{infNalicht}", String(bool_var_NachtLicht_an).c_str());
  root_content.replace("{infNoLicht}", String(bool_var_Notlicht_an).c_str());
  root_content.replace("{ssmqqb}", var_char_mqtt_server);
  root_content.replace("{smqqp}", String(var_int_mqtt_port));
  root_content.replace("{smqqln}", var_char_mqtt_user);
  root_content.replace("{smqqt}", str_fkt_topicpath());
  root_content.replace("{snwlssid}", var_char_mqtt_user);
  root_content.replace("{snwlIP}", String(WiFi.localIP()).c_str());
  server.send(200, "text/html", root_content);
  Serial.println("*INFO:DONE:Öffne RootWebseite");
  Serial.println("*----------------------------------");
}


/*
   ------------------------------------------------------------------------------------------------
   Looooooooping!
   ------------------------------------------------------------------------------------------------
*/
void loop() {
  Serial.println("############################################################################");
  Serial.println("*INFO:DO  :RUN LOOP START JETZT");
      String TMP_string_var_Temperatur = String(float_fkt_getTemp());
      String TMP_string_var_Luftfeuchtigkeit = String(float_fkt_getHum());
      String TMP_string_var_Umgebungslicht = String(float_fkt_getLicht());
      String TMP_string_var_Bewegungsmelder = String(bool_fkt_getMotion());
      String TMP_string_var_WLANProzent = String(int_fkt_WLANProzent(WiFi.RSSI()));
  
  while (!client.connected()) {
      Serial.println("*INFO:DO  :MQTT CONNECT");
      Serial.println("Connecting to MQTT...");//
      if (client.connect(WiFi.macAddress().c_str(), var_char_mqtt_user, var_char_mqtt_password )) {
        Serial.println("connected");
        Serial.println("*INFO:DONE:CHECK MQTT CONNECTION - OK");
      } else {
        Serial.print("failed with state ");
        Serial.print(client.state());
        delay(2000);
      }
    }
  

    if (client.connected()) {
      client.setCallback(callback);
      Serial.println("*INFO:DO  :CHECK MQTT PUSH / GET NEUE WERTE");

            
      if (string_var_Temperatur != TMP_string_var_Temperatur ){
        string_var_Temperatur = TMP_string_var_Temperatur;
        client.publish(Value1.c_str(), string_var_Temperatur.c_str());
      }

      if (string_var_Luftfeuchtigkeit != TMP_string_var_Luftfeuchtigkeit){
        string_var_Luftfeuchtigkeit = TMP_string_var_Luftfeuchtigkeit;
        client.publish(Value2.c_str(), string_var_Luftfeuchtigkeit.c_str());
      }

      if (string_var_Umgebungslicht != TMP_string_var_Umgebungslicht){
        string_var_Umgebungslicht = TMP_string_var_Umgebungslicht;
        client.publish(Value3.c_str(), string_var_Umgebungslicht.c_str() );
      }

      if (string_var_Bewegungsmelder != TMP_string_var_Bewegungsmelder){
        string_var_Bewegungsmelder = TMP_string_var_Bewegungsmelder;
        client.publish(Value4.c_str(), string_var_Bewegungsmelder.c_str() );
      }

      if ( String(int_fkt_WLANProzent(WiFi.RSSI())) != TMP_string_var_WLANProzent){
        TMP_string_var_Bewegungsmelder = String(int_fkt_WLANProzent(WiFi.RSSI()));
        client.publish(Value15.c_str(), String(int_fkt_WLANProzent(WiFi.RSSI())).c_str());
      }
      //client.loop();  

      
      //client.publish(Value5.c_str(), "90%" );
      //client.publish(Value6.c_str(), String(WiFi.localIP()).c_str());
      //client.publish(Value7.c_str(), "0" );
      //client.publish(Value8.c_str(), "0" );
      
      //Serial.println(client.subscribe(Value10.c_str()));  // optischer Alarm
      //client.subscribe(Value11.c_str());  // akkustuscher Alarm
      //client.subscribe(Value14.c_str());  // Akkubetrieb
      Serial.println("*INFO:DONE:CHECK MQTT PUSH / GET NEUE WERTE");
    }


  server.handleClient();
  delay(10000);
  Serial.println("############################################################################");
}

